@@ -185,9 +185,6 @@ grl_lua_factory_plugin_init (GrlRegistry *registry,
   GError *err = NULL;
   gboolean source_loaded = FALSE;
   GCancellable *cancellable;
-#ifdef GOA_ENABLED
-  GList *lua_init_sources = NULL;
-#endif
 
   GRL_LOG_DOMAIN_INIT (lua_factory_log_domain, "lua-factory");
 
@@ -220,11 +217,9 @@ grl_lua_factory_plugin_init (GrlRegistry *registry,
 
     goa_client_new (cancellable, grl_lua_factory_goa_init, goa_data);
 
-    lua_init_sources = g_list_prepend (lua_init_sources, goa_data);
   }
 
   g_list_free (goa_sources);
-  g_object_set_data (G_OBJECT (plugin), "lua-init-sources", lua_init_sources);
 #else
   g_assert (goa_sources == NULL);
 #endif
@@ -601,6 +596,8 @@ lua_plugin_source_init (GrlLuaFactorySource *lua_source)
     lua_pushnil (L);
   }
 
+  /* FIXME in order to supported async calls we should use
+   * grl_lua_operations_pcall */
   if (lua_pcall (L, 1, 1, 0)) {
     GRL_WARNING ("calling grl_source_init function failed: %s",
                  lua_tolstring (L, -1, NULL));
@@ -782,6 +779,7 @@ grl_lua_goa_data_free (GrlLuaGoaData *data)
   g_free (data->account_provider);
   g_free (data->account_feature);
   g_clear_pointer (&data->sources, g_hash_table_destroy);
+  g_free (data);
 }
 
 static void
@@ -793,6 +791,7 @@ grl_lua_factory_goa_init (GObject *source_object,
   GList *tmp;
   GList *acc_list;
   GList *lua_acc_list = NULL;
+  GList *lua_init_sources = NULL;
   GrlLuaGoaData *lua_data = user_data;
   GoaClient *client;
 
@@ -806,6 +805,10 @@ grl_lua_factory_goa_init (GObject *source_object,
     return;
   }
 
+  lua_init_sources = g_object_get_data (G_OBJECT (lua_data->plugin), "lua-init-sources");
+  lua_init_sources = g_list_prepend (lua_init_sources, lua_data);
+  g_object_set_data (G_OBJECT (lua_data->plugin), "lua-init-sources", lua_init_sources);
+
   lua_data->client = client;
 
   acc_list = goa_client_get_accounts (client);
@@ -891,7 +894,8 @@ keys_table_array_to_list (lua_State *L,
     if (key_id != GRL_METADATA_KEY_INVALID) {
       filtered_list = g_list_prepend (filtered_list, GRLKEYID_TO_POINTER (key_id));
     } else {
-      GRL_WARNING ("Unknown key '%s' in property '%s' for source '%s'", key_name, array_name, source_id);
+      GRL_DEBUG ("Unknown key '%s' in property '%s' for source '%s'",
+                 key_name, array_name, source_id);
     }
   }
   g_list_free_full (list, g_free);
@@ -1192,6 +1196,7 @@ merge_all_configs (const gchar *source_id,
     for (it_keys = list_keys; it_keys; it_keys = g_list_next (it_keys)) {
       options = g_list_append (options, g_strdup (it_keys->data));
     }
+    g_list_free(list_keys);
   }
 
   /* Now merge them */
@@ -1402,6 +1407,14 @@ lua_plugin_source_all_keys (lua_State *L,
         supported_media = GRL_SUPPORTED_MEDIA_IMAGE;
       else if (g_strcmp0 (key_name, "all") == 0)
         supported_media = GRL_SUPPORTED_MEDIA_ALL;
+
+      if (supported_media == GRL_SUPPORTED_MEDIA_NONE) {
+        GRL_WARNING ("(%s) value '%s' is not supported on %s.type ",
+                     source_id, key_name, LUA_SOURCE_RESOLVE_KEYS);
+      }
+    } else {
+      GRL_WARNING ("(%s) expecting string on %s.type but got instead %s", source_id,
+                   LUA_SOURCE_RESOLVE_KEYS, lua_typename (L, lua_type (L, -1)));
     }
     lua_pop (L, 1);
 
@@ -1409,6 +1422,9 @@ lua_plugin_source_all_keys (lua_State *L,
 
     /* check required table field */
     *resolve_keys = keys_table_array_to_list (L, LUA_REQUIRED_TABLE, registry, source_id);
+  } else if (!lua_isnil (L, -1)) {
+    GRL_WARNING ("(%s) expecting table on %s but got instead %s", source_id,
+                 LUA_SOURCE_RESOLVE_KEYS, lua_typename (L, lua_type (L, -1)));
   }
   lua_pop (L, 1);
 
@@ -1438,7 +1454,17 @@ lua_plugin_source_all_keys (lua_State *L,
     }
     g_list_free_full (table_list, g_free);
 
+    if (g_hash_table_size (htable) == 0) {
+      GRL_WARNING ("(%s) at %s - no valid config-keys on %s and %s", source_id,
+                   LUA_SOURCE_CONFIG_KEYS, LUA_REQUIRED_TABLE, LUA_OPTIONAL_TABLE);
+      g_hash_table_destroy (htable);
+      htable = NULL;
+    }
+
     *config_keys = htable;
+  } else if (!lua_isnil (L, -1)) {
+    GRL_WARNING ("(%s) expecting table on %s but got instead %s", source_id,
+                 LUA_SOURCE_CONFIG_KEYS, lua_typename (L, lua_type (L, -1)));
   }
   lua_pop (L, 1);
 
@@ -1483,6 +1509,7 @@ grl_lua_factory_source_search (GrlSource *source,
   lua_State *L = lua_source->priv->l_st;
   OperationSpec *os = NULL;
   const gchar *text = NULL;
+  GError *err = NULL;
 
   GRL_DEBUG ("grl_lua_factory_source_search");
 
@@ -1499,18 +1526,15 @@ grl_lua_factory_source_search (GrlSource *source,
   os->options = grl_operation_options_copy (ss->options);
   os->op_type = LUA_SEARCH;
 
-  grl_lua_library_save_operation_data (L, os);
-  grl_lua_library_set_current_operation (L, os->operation_id);
   lua_getglobal (L, LUA_SOURCE_OPERATION[LUA_SEARCH]);
 
   lua_pushstring (L, text);
-  if (lua_pcall (L, 1, 0, 0)) {
-    GRL_WARNING ("%s '%s'", "calling search function fail:",
-                 lua_tolstring (L, -1, NULL));
-    lua_pop (L, 1);
+  if (!grl_lua_operations_pcall (L, 1, os, &err)) {
+    if (err != NULL) {
+      GRL_WARNING ("calling search function failed: %s", err->message);
+      g_error_free (err);
+    }
   }
-
-  grl_lua_library_set_current_operation (L, 0);
 }
 
 static void
@@ -1520,6 +1544,7 @@ grl_lua_factory_source_browse (GrlSource *source,
   GrlLuaFactorySource *lua_source = GRL_LUA_FACTORY_SOURCE (source);
   lua_State *L = lua_source->priv->l_st;
   OperationSpec *os = NULL;
+  GError *err = NULL;
   const gchar *media_id = NULL;
 
   GRL_DEBUG ("grl_lua_factory_source_browse");
@@ -1538,18 +1563,15 @@ grl_lua_factory_source_browse (GrlSource *source,
   os->options = grl_operation_options_copy (bs->options);
   os->op_type = LUA_BROWSE;
 
-  grl_lua_library_save_operation_data (L, os);
-  grl_lua_library_set_current_operation (L, os->operation_id);
   lua_getglobal (L, LUA_SOURCE_OPERATION[LUA_BROWSE]);
 
   lua_pushstring (L, media_id);
-  if (lua_pcall (L, 1, 0, 0)) {
-    GRL_WARNING ("%s '%s'", "calling browse function fail:",
-                 lua_tolstring (L, -1, NULL));
-    lua_pop (L, 1);
+  if (!grl_lua_operations_pcall (L, 1, os, &err)) {
+    if (err != NULL) {
+      GRL_WARNING ("calling browse function failed: %s", err->message);
+      g_error_free (err);
+    }
   }
-
-  grl_lua_library_set_current_operation (L, 0);
 }
 
 static void
@@ -1560,6 +1582,7 @@ grl_lua_factory_source_query (GrlSource *source,
   lua_State *L = lua_source->priv->l_st;
   OperationSpec *os = NULL;
   const gchar *query = NULL;
+  GError *err = NULL;
 
   GRL_DEBUG ("grl_lua_factory_source_query");
 
@@ -1576,18 +1599,15 @@ grl_lua_factory_source_query (GrlSource *source,
   os->options = grl_operation_options_copy (qs->options);
   os->op_type = LUA_QUERY;
 
-  grl_lua_library_save_operation_data (L, os);
-  grl_lua_library_set_current_operation (L, os->operation_id);
   lua_getglobal (L, LUA_SOURCE_OPERATION[LUA_QUERY]);
 
   lua_pushstring (L, query);
-  if (lua_pcall (L, 1, 0, 0)) {
-    GRL_WARNING ("%s '%s'", "calling query function fail:",
-                 lua_tolstring (L, -1, NULL));
-    lua_pop (L, 1);
+  if (!grl_lua_operations_pcall (L, 1, os, &err)) {
+    if (err != NULL) {
+      GRL_WARNING ("calling query function failed: %s", err->message);
+      g_error_free (err);
+    }
   }
-
-  grl_lua_library_set_current_operation (L, 0);
 }
 
 static void
@@ -1597,11 +1617,10 @@ grl_lua_factory_source_resolve (GrlSource *source,
   GrlLuaFactorySource *lua_source = GRL_LUA_FACTORY_SOURCE (source);
   lua_State *L = lua_source->priv->l_st;
   OperationSpec *os = NULL;
+  GError *err = NULL;
 
   GRL_DEBUG ("grl_lua_factory_source_resolve");
 
-  g_return_if_fail (grl_lua_library_load_operation_data (L, rs->operation_id) == NULL);
-
   os = g_slice_new0 (OperationSpec);
   os->source = rs->source;
   os->operation_id = rs->operation_id;
@@ -1613,17 +1632,14 @@ grl_lua_factory_source_resolve (GrlSource *source,
   os->options = grl_operation_options_copy (rs->options);
   os->op_type = LUA_RESOLVE;
 
-  grl_lua_library_save_operation_data (L, os);
-  grl_lua_library_set_current_operation (L, os->operation_id);
   lua_getglobal (L, LUA_SOURCE_OPERATION[LUA_RESOLVE]);
 
-  if (lua_pcall (L, 0, 0, 0)) {
-    GRL_WARNING ("%s '%s'", "calling resolve function fail:",
-                 lua_tolstring (L, -1, NULL));
-    lua_pop (L, 1);
+  if (!grl_lua_operations_pcall (L, 0, os, &err)) {
+    if (err != NULL) {
+      GRL_WARNING ("calling resolve function failed: %s", err->message);
+      g_error_free (err);
+    }
   }
-
-  grl_lua_library_set_current_operation (L, 0);
 }
 
 static gboolean
@@ -1655,7 +1671,7 @@ grl_lua_factory_source_may_resolve (GrlSource *source,
 
   /* Verify if the source resolve type and media type match */
   res_type = lua_source->priv->resolve_type;
-  if ((grl_media_is_container (media) && (res_type != GRL_SUPPORTED_MEDIA_ALL))
+  if ((grl_media_is_container (media) && !(res_type & GRL_SUPPORTED_MEDIA_ALL))
       || (grl_media_is_audio (media) && !(res_type & GRL_SUPPORTED_MEDIA_AUDIO))
       || (grl_media_is_image (media) && !(res_type & GRL_SUPPORTED_MEDIA_IMAGE))
       || (grl_media_is_video (media) && !(res_type & GRL_SUPPORTED_MEDIA_VIDEO))) {
