@@ -38,26 +38,31 @@
 #include <totem-pl-parser-mini.h>
 #endif
 
+#define LUA_SOURCE_TABLE            "source"
+#define LUA_SOURCE_TAGS             "tags"
+
 #define GRL_LOG_DOMAIN_DEFAULT lua_library_log_domain
 GRL_LOG_DOMAIN_STATIC (lua_library_log_domain);
 
 typedef struct {
   lua_State *L;
-  guint operation_id;
-  gchar *lua_cb;
+  gint lua_userdata;
+  gint lua_callback;
   guint index;
   gchar *url;
   guint num_urls;
   gboolean is_table;
   gchar **results;
+  OperationSpec *os;
 } FetchOperation;
 
 typedef struct {
   lua_State *L;
-  guint operation_id;
-  gchar *lua_cb;
+  gint lua_userdata;
+  gint lua_callback;
   gchar *url;
   gchar **filenames;
+  OperationSpec *os;
 } UnzipOperation;
 
 /* ================== Lua-Library utils/helpers ============================ */
@@ -219,21 +224,35 @@ grl_data_add_lua_string (GrlData    *data,
 }
 
 static gboolean
-verify_plaintext_fetch (GrlSource  *source,
-                        char      **urls,
+verify_plaintext_fetch (lua_State  *L,
+                        gchar     **urls,
                         guint       num_urls)
 {
-  const char **tags;
-  gboolean has_plaintext_tag;
   guint i;
 
-  tags = grl_source_get_tags (source);
-  has_plaintext_tag = (tags && g_strv_contains (tags, "net:plaintext"));
+  lua_getglobal (L, LUA_SOURCE_TABLE);
+  if (!lua_istable (L, -1)) {
+    lua_pop (L, 1);
+    return FALSE;
+  }
+  lua_getfield (L, -1, LUA_SOURCE_TAGS);
+  if (!lua_istable (L, -1)) {
+    lua_pop (L, 2);
+    return FALSE;
+  }
 
-  /* No need to verify the URLs, the source is saying that they do
-   * plaintext queries, so nothing for us to block */
-  if (has_plaintext_tag)
-    return TRUE;
+  lua_pushnil (L);
+  while (lua_next (L, -2) != 0) {
+    if (g_strcmp0 (lua_tostring (L, -1), "net:plaintext") == 0) {
+      /* No need to verify the URLs, the source is saying that they do
+       * plaintext queries, so nothing for us to block */
+      lua_pop (L, 4);
+      return TRUE;
+    }
+    lua_pop (L, 1);
+  }
+
+  lua_pop (L, 2);
 
   for (i = 0; i < num_urls; i++) {
     if (g_str_has_prefix (urls[i], "http:"))
@@ -451,9 +470,9 @@ grl_util_fetch_done (GObject *source_object,
   gsize len;
   guint i;
   GError *err = NULL;
-  OperationSpec *os;
   FetchOperation *fo = (FetchOperation *) user_data;
   lua_State *L = fo->L;
+  OperationSpec *os = fo->os;
   gchar *fixed = NULL;
 
   if (!grl_net_wc_request_finish (GRL_NET_WC (source_object),
@@ -485,17 +504,14 @@ grl_util_fetch_done (GObject *source_object,
     if (fo->results[i] == NULL) {
       /* Clean up this operation, and wait for
        * other operations to complete */
-      g_free (fo->lua_cb);
+      g_free (fo->url);
       g_free (fo);
       return;
     }
   }
 
-  grl_lua_library_set_current_operation (L, fo->operation_id);
-  os = grl_lua_library_get_current_operation (L);
-  os->pending_ops--;
-
-  lua_getglobal (L, fo->lua_cb);
+  /* get the callback from the registry */
+  lua_rawgeti (L, LUA_REGISTRYINDEX, fo->lua_callback);
 
   if (!fo->is_table) {
     lua_pushlstring (L, fo->results[0], strlen (fo->results[0]));
@@ -508,18 +524,23 @@ grl_util_fetch_done (GObject *source_object,
     }
   }
 
-  if (lua_pcall (L, 1, 0, 0)) {
-    GRL_WARNING ("%s (%s) '%s'", "calling source callback function fail",
-                 fo->lua_cb, lua_tolstring (L, -1, NULL));
+  /* get userdata from the registry */
+  lua_rawgeti (L, LUA_REGISTRYINDEX, fo->lua_userdata);
+
+  if (!grl_lua_operations_pcall (L, 2, os, &err)) {
+    if (err != NULL) {
+      GRL_WARNING ("calling source callback function fail: %s", err->message);
+      g_error_free (err);
+    }
   }
 
-  grl_lua_library_set_current_operation (L, 0);
+  luaL_unref (L, LUA_REGISTRYINDEX, fo->lua_userdata);
+  luaL_unref (L, LUA_REGISTRYINDEX, fo->lua_callback);
 
   for (i = 0; i < fo->num_urls; i++)
     g_free (fo->results[i]);
   g_free (fo->url);
   g_free (fo->results);
-  g_free (fo->lua_cb);
   g_free (fo);
 }
 
@@ -610,9 +631,9 @@ grl_util_unzip_done (GObject *source_object,
   gsize len;
   guint i;
   GError *err = NULL;
-  OperationSpec *os;
   UnzipOperation *uo = (UnzipOperation *) user_data;
   lua_State *L = uo->L;
+  OperationSpec *os = uo->os;
   char **results;
 
   grl_net_wc_request_finish (GRL_NET_WC (source_object),
@@ -631,11 +652,8 @@ grl_util_unzip_done (GObject *source_object,
     results = get_zipped_contents ((guchar *) data, len, (const gchar **) uo->filenames);
   }
 
-  grl_lua_library_set_current_operation (L, uo->operation_id);
-  os = grl_lua_library_get_current_operation (L);
-  os->pending_ops--;
-
-  lua_getglobal (L, uo->lua_cb);
+  /* get the callback from the registry */
+  lua_rawgeti(L, LUA_REGISTRYINDEX, uo->lua_callback);
 
   lua_newtable (L);
   for (i = 0; results[i] != NULL; i++) {
@@ -644,24 +662,29 @@ grl_util_unzip_done (GObject *source_object,
     lua_settable (L, -3);
   }
 
-  if (lua_pcall (L, 1, 0, 0)) {
-    GRL_WARNING ("%s (%s) '%s'", "calling source callback function fail",
-                 uo->lua_cb, lua_tolstring (L, -1, NULL));
+  /* get userdata from the registry */
+  lua_rawgeti (L, LUA_REGISTRYINDEX, uo->lua_userdata);
+
+  if (!grl_lua_operations_pcall (L, 2, os, &err)) {
+    if (err != NULL) {
+      GRL_WARNING ("calling source callback function fail: %s", err->message);
+      g_error_free (err);
+    }
   }
 
-  grl_lua_library_set_current_operation (L, 0);
+  luaL_unref (L, LUA_REGISTRYINDEX, uo->lua_userdata);
+  luaL_unref (L, LUA_REGISTRYINDEX, uo->lua_callback);
 
   g_strfreev (results);
 
   g_strfreev (uo->filenames);
-  g_free (uo->lua_cb);
   g_free (uo->url);
   g_free (uo);
 }
 
 static GrlNetWc *
-net_wc_new_with_options(lua_State *L,
-                        guint      arg_offset)
+net_wc_new_with_options (lua_State *L,
+                         guint      arg_offset)
 {
   GrlNetWc *wc;
 
@@ -720,7 +743,7 @@ grl_l_operation_get_options (lua_State *L)
 
   luaL_argcheck (L, lua_isstring (L, 1), 1, "expecting option (string)");
 
-  os = grl_lua_library_get_current_operation (L);
+  os = grl_lua_operations_get_current_op (L);
   g_return_val_if_fail (os != NULL, 0);
   option = lua_tostring (L, 1);
 
@@ -878,25 +901,30 @@ grl_l_operation_get_keys (lua_State *L)
   OperationSpec *os;
   GrlRegistry *registry;
   GList *it;
-  gint i = 0;
 
-  os = grl_lua_library_get_current_operation (L);
+  os = grl_lua_operations_get_current_op (L);
   g_return_val_if_fail (os != NULL, 0);
 
   registry = grl_registry_get_default ();
   lua_newtable (L);
-  for (it = os->keys; it; it = g_list_next (it)) {
+  for (it = os->keys; it != NULL; it = it->next) {
     GrlKeyID key_id;
-    const gchar *key_name;
+    char *key_name, *ptr;
 
     key_id = GRLPOINTER_TO_KEYID (it->data);
-    key_name = grl_registry_lookup_metadata_key_name (registry, key_id);
-    if (key_id != GRL_METADATA_KEY_INVALID) {
-      lua_pushinteger (L, i + 1);
-      lua_pushstring (L, key_name);
-      lua_settable (L, -3);
-      i++;
+    if (key_id == GRL_METADATA_KEY_INVALID)
+      continue;
+
+    key_name = g_strdup (grl_registry_lookup_metadata_key_name (registry, key_id));
+    /* Replace '-' to '_': convenient for the developer */
+    while ((ptr = strstr (key_name, "-")) != NULL) {
+      *ptr = '_';
     }
+
+    lua_pushstring (L, key_name);
+    lua_pushboolean (L, 1);
+    lua_settable (L, -3);
+    g_free (key_name);
   }
   return 1;
 }
@@ -999,13 +1027,37 @@ grl_l_media_get_keys (lua_State *L)
   GrlRegistry *registry;
   GList *it;
   GList *list_keys;
+  const gchar *media_type = NULL;
+  GrlMedia *media;
 
-  os = grl_lua_library_get_current_operation (L);
-  g_return_val_if_fail (os != NULL, 0);
+  os = grl_lua_operations_get_current_op (L);
+  media = os->media;
+
+  if (media == NULL) {
+    lua_pushnil (L);
+    return 1;
+  }
 
   registry = grl_registry_get_default ();
   lua_newtable (L);
-  list_keys = grl_data_get_keys (GRL_DATA (os->media));
+
+  if (grl_media_is_audio (media)) {
+    media_type = "audio";
+  } else if (grl_media_is_video (media)) {
+    media_type = "video";
+  } else if (grl_media_is_image (media)) {
+    media_type = "image";
+  } else if (grl_media_is_container (media)) {
+    media_type = "container";
+  }
+
+  if (media_type) {
+    lua_pushstring (L, "type");
+    lua_pushstring (L, media_type);
+    lua_settable (L, -3);
+  }
+
+  list_keys = grl_data_get_keys (GRL_DATA (media));
   for (it = list_keys; it != NULL; it = it->next) {
     GrlKeyID key_id;
     gchar *key_name;
@@ -1023,7 +1075,7 @@ grl_l_media_get_keys (lua_State *L)
     }
 
     lua_pushstring (L, key_name);
-    if (push_grl_media_key (L, os->media, key_id))
+    if (push_grl_media_key (L, media, key_id))
       lua_settable (L, -3);
     else
       lua_pop (L, 1);
@@ -1038,8 +1090,9 @@ grl_l_media_get_keys (lua_State *L)
 * grl.fetch
 *
 * @url: (string or array) The http url to GET the content.
-* @callback: (string) The function to be called after fetch is complete.
-* @netopts: (table) Options to set the GrlNetWc object.
+* @netopts: [optional] (table) Options to set the GrlNetWc object.
+* @callback: (function) The function to be called after fetch is complete.
+* @userdata: [optional] User data to be passed to the @callback.
 * @return: Nothing.;
 */
 static gint
@@ -1049,22 +1102,39 @@ grl_l_fetch (lua_State *L)
   guint num_urls;
   gchar **urls;
   gchar **results;
-  const gchar *lua_callback;
+  gint lua_userdata;
+  gint lua_callback;
   GrlNetWc *wc;
   gboolean is_table = FALSE;
-  gboolean wc_options = FALSE;
   OperationSpec *os;
 
   luaL_argcheck (L, (lua_isstring (L, 1) || lua_istable (L, 1)), 1,
                  "expecting url as string or an array of urls");
-  luaL_argcheck (L, lua_isstring (L, 2), 2,
-                 "expecting callback function as string");
-  /* Optional third argument */
-  wc_options = (lua_gettop(L) == 3);
+  luaL_argcheck (L, (lua_isfunction (L, 2) || lua_istable (L, 2)), 2,
+                 "expecting callback function or network parameters");
 
-  os = grl_lua_library_get_current_operation (L);
-  g_return_val_if_fail (os != NULL, 0);
-  os->pending_ops++;
+  luaL_argcheck (L, (lua_isfunction (L, 2) ||
+                     (lua_istable (L, 2) && lua_isfunction (L, 3))), 3,
+                 "expecting callback function after network parameters");
+
+  os = grl_lua_operations_get_current_op (L);
+
+  /* keep arguments aligned */
+  if (lua_isfunction (L, 2)) {
+    lua_pushnil (L);
+    lua_insert (L, 2);
+  }
+
+  if (lua_gettop (L) > 4)
+    luaL_error (L, "too many arguments to 'fetch' function");
+
+  /* add nil if userdata is omitted */
+  lua_settop (L, 4);
+
+  /* pop the userdata and store it in registry */
+  lua_userdata = luaL_ref (L, LUA_REGISTRYINDEX);
+  /* pop the callback and store it in registry */
+  lua_callback = luaL_ref (L, LUA_REGISTRYINDEX);
 
   num_urls = (lua_isstring (L, 1)) ? 1 : luaL_len (L, 1);
   urls = g_new0 (gchar *, num_urls);
@@ -1088,17 +1158,18 @@ grl_l_fetch (lua_State *L)
     }
   }
 
-  if (!verify_plaintext_fetch (os->source, urls, num_urls)) {
+  if (!verify_plaintext_fetch (L, urls, num_urls)) {
     GRL_WARNING ("Source '%s' is broken, it makes plaintext network queries but "
                  "does not set the 'net:plaintext' tag", grl_source_get_id (os->source));
+
+    luaL_unref (L, LUA_REGISTRYINDEX, lua_userdata);
+    luaL_unref (L, LUA_REGISTRYINDEX, lua_callback);
+    lua_gc (L, LUA_GCCOLLECT, 0);
     g_free (urls);
-    os->pending_ops--;
-    return 1;
+    return 0;
   }
 
-  lua_callback = lua_tolstring (L, 2, NULL);
-
-  wc = (wc_options) ? net_wc_new_with_options(L, 3) : grl_net_wc_new ();
+  wc = net_wc_new_with_options (L, 2);
 
   /* shared data between urls */
   results = g_new0 (gchar *, num_urls);
@@ -1107,8 +1178,9 @@ grl_l_fetch (lua_State *L)
 
     fo = g_new0 (FetchOperation, 1);
     fo->L = L;
-    fo->operation_id = os->operation_id;
-    fo->lua_cb = g_strdup (lua_callback);
+    fo->os = os;
+    fo->lua_userdata = lua_userdata;
+    fo->lua_callback = lua_callback;
     fo->index = i;
     fo->url = g_strdup (urls[i]);
     fo->num_urls = num_urls;
@@ -1119,7 +1191,10 @@ grl_l_fetch (lua_State *L)
   }
   g_object_unref (wc);
   g_free (urls);
-  return 1;
+
+  /* Set the state as wating for this async operation */
+  grl_lua_operations_set_source_state (L, LUA_SOURCE_WAITING, os);
+  return 0;
 }
 
 /**
@@ -1140,14 +1215,15 @@ grl_l_callback (lua_State *L)
   GRL_DEBUG ("grl.callback()");
 
   nparam = lua_gettop (L);
-  os = grl_lua_library_get_current_operation (L);
+  os = grl_lua_operations_get_current_op (L);
   if (os == NULL) {
-    luaL_error (L, "Source is broken as grl.callback() was called "
-                "after last operation has been finalized");
+    luaL_error (L, "Source is broken as callback was called "
+                "after the operation has been finalized");
     return 0;
   }
 
   media = (os->op_type == LUA_RESOLVE) ? os->media : NULL;
+
   if (nparam > 0) {
     media = grl_util_build_media (L, media);
     count = (lua_isinteger (L, 2)) ? lua_tointeger (L, 2) : 0;
@@ -1163,16 +1239,9 @@ grl_l_callback (lua_State *L)
                    count, os->user_data, NULL);
   }
 
-  /* Free Operation Spec */
-  if (count == 0) {
-    g_list_free (os->keys);
-    g_object_unref (os->options);
-    os->callback_done = TRUE;
-    grl_lua_library_remove_operation_data (L, os->operation_id);
-    grl_lua_library_set_current_operation (L, 0);
-    g_free (os->string);
-    g_slice_free (OperationSpec, os);
-  }
+  /* finishing callback */
+  if (count == 0)
+    grl_lua_operations_set_source_state (L, LUA_SOURCE_FINALIZED, os);
 
   return 0;
 }
@@ -1314,31 +1383,49 @@ grl_l_unescape (lua_State *L)
  *
  * @url: (string) the URL of the zip file to fetch
  * @filenames: (table) a table of filenames to get inside the zip file
- * @callback: (string) The function to be called after fetch is complete.
- * @netopts: (table) Options to set the GrlNetWc object.
+ * @netopts: [optional] (table) Options to set the GrlNetWc object.
+ * @callback: (function) The function to be called after fetch is complete.
+ * @userdata: [optional] User data to be passed to the @callback.
  * @return: Nothing.;
  */
 static gint
 grl_l_unzip (lua_State *L)
 {
-  const gchar *lua_callback;
+  gint lua_userdata;
+  gint lua_callback;
   const gchar *url;
   GrlNetWc *wc;
-  OperationSpec *os;
   UnzipOperation *uo;
   guint num_filenames, i;
   gchar **filenames;
+  OperationSpec *os;
 
   luaL_argcheck (L, lua_isstring (L, 1), 1,
                  "expecting url as string");
   luaL_argcheck (L, lua_istable (L, 2), 2,
                  "expecting filenames as an array of filenames");
-  luaL_argcheck (L, lua_isstring (L, 3), 3,
-                 "expecting callback function as string");
+  luaL_argcheck (L, (lua_isfunction (L, 3) || lua_istable (L, 3)), 3,
+                 "expecting callback function or network parameters");
+  luaL_argcheck (L, (lua_isfunction (L, 3) ||
+                     (lua_istable (L, 3) && lua_isfunction (L, 4))), 4,
+                 "expecting callback function after network parameters");
 
-  os = grl_lua_library_get_current_operation (L);
-  g_return_val_if_fail (os != NULL, 0);
-  os->pending_ops++;
+  /* keep arguments aligned */
+  if (lua_isfunction (L, 3)) {
+    lua_pushnil (L);
+    lua_insert (L, 3);
+  }
+
+  if (lua_gettop (L) > 5)
+    luaL_error (L, "too many arguments to 'unzip' function");
+
+  /* add nil if userdata is omitted */
+  lua_settop (L, 5);
+
+  /* pop the userdata and store it in registry */
+  lua_userdata = luaL_ref (L, LUA_REGISTRYINDEX);
+  /* pop the callback and store it in registry */
+  lua_callback = luaL_ref (L, LUA_REGISTRYINDEX);
 
   url = lua_tolstring (L, 1, NULL);
   num_filenames = luaL_len(L, 2);
@@ -1357,19 +1444,22 @@ grl_l_unzip (lua_State *L)
   }
   GRL_DEBUG ("grl.unzip() -> '%s'", url);
 
-  lua_callback = lua_tolstring (L, 3, NULL);
-  wc = net_wc_new_with_options (L, 4);
+  wc = net_wc_new_with_options (L, 3);
+  os = grl_lua_operations_get_current_op (L);
 
   uo = g_new0 (UnzipOperation, 1);
   uo->L = L;
-  uo->operation_id = os->operation_id;
-  uo->lua_cb = g_strdup (lua_callback);
+  uo->lua_userdata = lua_userdata;
+  uo->lua_callback = lua_callback;
   uo->url = g_strdup (url);
   uo->filenames = filenames;
+  uo->os = os;
 
   grl_net_wc_request_async (wc, url, NULL, grl_util_unzip_done, uo);
   g_object_unref (wc);
-  return 1;
+
+  grl_lua_operations_set_source_state (L, LUA_SOURCE_WAITING, os);
+  return 0;
 }
 
 /**
@@ -1563,153 +1653,22 @@ luaopen_grilo (lua_State *L)
   }
   lua_pop (L, 1);
 
+  /* grl.lua will be read-only from now on for Lua sources */
+  grl_lua_operations_set_proxy_table (L, -1);
+
   /* Those modules are called in 'lua' table, inside 'grl' */
   lua_settable (L, -3);
+
+  grl_lua_operations_init_priv_state (L);
+
+  /* grl will be read-only from now on for Lua sources */
+  grl_lua_operations_set_proxy_table (L, -1);
+
   return 1;
 }
 
 /* ======= Lua-Library and Lua-Factory utilities ============= */
 
-/**
- * grl_lua_library_save_operation_data
- *
- * @L : LuaState where the data will be stored.
- * @os: The Operation Data to store.
- * @return: Nothing.
- *
- * Stores the OperationSpec from Lua-Factory in the global environment of
- * lua_State.
- **/
-void
-grl_lua_library_save_operation_data (lua_State *L, OperationSpec *os)
-{
-  char *op_id;
-
-  g_return_if_fail (os != NULL);
-
-  op_id = g_strdup_printf (GRILO_LUA_OPERATION_INDEX "-%i", os->operation_id);
-  lua_getglobal (L, LUA_ENV_TABLE);
-  lua_pushstring (L, op_id);
-  lua_pushlightuserdata (L, os);
-  lua_settable (L, -3);
-  lua_pop (L, 1);
-  g_free (op_id);
-}
-
-/**
- * grl_lua_library_remove_operation_data
- *
- * @L: LuaState where the data will be removed.
- * @operation_id: The operation ID to remove.
- * @return: Nothing.
- *
- * Remove the OperationSpec with this ID from the the global environment of
- * lua_State.
- **/
-void
-grl_lua_library_remove_operation_data (lua_State *L, guint operation_id)
-{
-  char *op_id;
-
-  op_id = g_strdup_printf (GRILO_LUA_OPERATION_INDEX "-%i", operation_id);
-  lua_getglobal (L, LUA_ENV_TABLE);
-  lua_pushstring (L, op_id);
-  lua_pushlightuserdata (L, NULL);
-  lua_settable (L, -3);
-  lua_pop (L, 1);
-  g_free (op_id);
-}
-
-/**
- * grl_lua_library_load_operation_data
- *
- * @L : LuaState where the data is stored.
- * @operation_id: The operation ID to load Operation Data for.
- * to load the Operation Data for the current call.
- * @return: The Operation Data.
- **/
-OperationSpec *
-grl_lua_library_load_operation_data (lua_State *L, guint operation_id)
-{
-  OperationSpec *os = NULL;
-  char *op_id;
-
-  g_return_val_if_fail (operation_id > 0, NULL);
-
-  op_id = g_strdup_printf (GRILO_LUA_OPERATION_INDEX "-%i", operation_id);
-  lua_getglobal (L, LUA_ENV_TABLE);
-  lua_pushstring (L, op_id);
-  lua_gettable (L, -2);
-  os = (lua_islightuserdata(L, -1)) ? lua_touserdata(L, -1) : NULL;
-  lua_pop(L, 1);
-  g_free (op_id);
-
-  return os;
-}
-
-/**
- * grl_lua_library_set_current_operation
- *
- * @L: LuaState where the data is stored.
- * @operation_id: The current operation ID.
- * @return: Nothing:
- **/
-void
-grl_lua_library_set_current_operation (lua_State *L, guint operation_id)
-{
-  OperationSpec *os;
-
-  /* Verify that either grl.callback was called, or that there
-   * are pending operations */
-  os = grl_lua_library_get_current_operation (L);
-  if (os) {
-    if (os->pending_ops == 0 && !os->callback_done) {
-      GRL_WARNING ("Source '%s' is broken, as there are no pending operations "
-                   "and grl.callback() was not called", grl_source_get_id (os->source));
-      switch (os->op_type) {
-      case LUA_RESOLVE:
-        os->cb.resolve (os->source, os->operation_id, NULL, os->user_data, NULL);
-        break;
-
-      default:
-        os->cb.result (os->source, os->operation_id, NULL,
-                       0, os->user_data, NULL);
-      }
-    }
-  }
-
-  if (operation_id > 0)
-    os = grl_lua_library_load_operation_data (L, operation_id);
-  else
-    os = NULL;
-
-  lua_getglobal (L, LUA_ENV_TABLE);
-  lua_pushstring (L, GRILO_LUA_OPERATION_INDEX);
-  lua_pushlightuserdata (L, os);
-  lua_settable (L, -3);
-  lua_pop (L, 1);
-}
-
-/**
- * grl_lua_library_get_current_operation
- *
- * @L: LuaState where the data is stored.
- * @return: The Operation Data for the current operation.
- **/
-OperationSpec *
-grl_lua_library_get_current_operation (lua_State *L)
-{
-  OperationSpec *os = NULL;
-
-  lua_getglobal (L, LUA_ENV_TABLE);
-  lua_pushstring (L, GRILO_LUA_OPERATION_INDEX);
-  lua_gettable (L, -2);
-  os = (lua_islightuserdata(L, -1)) ? lua_touserdata(L, -1) : NULL;
-  lua_pop(L, 1);
-
-  return os;
-}
-
 /**
  * grl_lua_library_save_goa_data
  *
