@@ -126,12 +126,13 @@ insert_key_mapping (GrlKeyID     grl_key,
                                            GRLKEYID_TO_POINTER (grl_key));
   gchar *canon_name = g_strdup (GRL_METADATA_KEY_GET_NAME (grl_key));
 
-  assoc->grl_key              = grl_key;
-  assoc->sparql_key_name      = build_flavored_key (canon_name,
-                                                    sparql_key_flavor);
-  assoc->sparql_key_attr      = sparql_key_attr;
-  assoc->sparql_key_attr_call = sparql_key_attr_call;
-  assoc->sparql_key_flavor    = sparql_key_flavor;
+  assoc->grl_key               = grl_key;
+  assoc->sparql_key_name       = build_flavored_key (canon_name,
+                                                     sparql_key_flavor);
+  assoc->sparql_key_name_canon = g_strdup (canon_name);
+  assoc->sparql_key_attr       = sparql_key_attr;
+  assoc->sparql_key_attr_call  = sparql_key_attr_call;
+  assoc->sparql_key_flavor     = sparql_key_flavor;
 
   assoc_list = g_list_append (assoc_list, assoc);
 
@@ -145,6 +146,16 @@ insert_key_mapping (GrlKeyID     grl_key,
                        (gpointer) GRL_METADATA_KEY_GET_NAME (grl_key),
                        assoc);
 
+  /* Grilo maps key names to SPARQL variables. Key names can contain dashes,
+   * however SPARQL does not allow dashes in variable names. So use the to
+   * underscores converted canon_name as additional mapping.
+   */
+  if (g_strrstr (assoc->sparql_key_name_canon, "_")) {
+    g_hash_table_insert (sparql_to_grl_mapping,
+                         (gpointer) assoc->sparql_key_name_canon,
+                         assoc);
+  }
+
   g_free (canon_name);
 
   return assoc;
@@ -568,12 +579,35 @@ grl_tracker_get_delete_conditional_string (const gchar *urn,
   return ret;
 }
 
+static GrlMedia *
+grl_tracker_build_grilo_media_default (GHashTable *ht)
+{
+  if (g_hash_table_lookup (ht, RDF_TYPE_MUSIC)) {
+    return grl_media_audio_new ();
+  } else if (g_hash_table_lookup (ht, RDF_TYPE_VIDEO)) {
+    return grl_media_video_new ();
+  } else if (g_hash_table_lookup (ht, RDF_TYPE_IMAGE)) {
+    return grl_media_image_new ();
+  } else if (g_hash_table_lookup (ht, RDF_TYPE_ARTIST)) {
+    return grl_media_container_new ();
+  } else if (g_hash_table_lookup (ht, RDF_TYPE_ALBUM)) {
+    return grl_media_container_new ();
+  } else if (g_hash_table_lookup (ht, RDF_TYPE_CONTAINER)) {
+    return grl_media_container_new ();
+  } else if (g_hash_table_lookup (ht, RDF_TYPE_FOLDER)) {
+    return grl_media_container_new ();
+  }
+
+  return NULL;
+}
+
 /**/
 
 /* Builds an appropriate GrlMedia based on ontology type returned by
    tracker, or NULL if unknown */
 GrlMedia *
-grl_tracker_build_grilo_media (const gchar *rdf_type)
+grl_tracker_build_grilo_media (const gchar   *rdf_type,
+                               GrlTypeFilter  type_filter)
 {
   GrlMedia *media = NULL;
   gchar **rdf_single_type;
@@ -591,20 +625,20 @@ grl_tracker_build_grilo_media (const gchar *rdf_type)
   for (; i>= 0; i--)
     g_hash_table_insert (ht, g_path_get_basename (rdf_single_type[i]), GINT_TO_POINTER(TRUE));
 
-  if (g_hash_table_lookup (ht, RDF_TYPE_MUSIC)) {
+  if (type_filter == GRL_TYPE_FILTER_NONE ||
+      type_filter == GRL_TYPE_FILTER_ALL) {
+    media = grl_tracker_build_grilo_media_default (ht);
+  } else if ((type_filter & GRL_TYPE_FILTER_AUDIO) &&
+             g_hash_table_lookup (ht, RDF_TYPE_MUSIC)) {
     media = grl_media_audio_new ();
-  } else if (g_hash_table_lookup (ht, RDF_TYPE_VIDEO)) {
+  } else if ((type_filter & GRL_TYPE_FILTER_VIDEO) &&
+             g_hash_table_lookup (ht, RDF_TYPE_VIDEO)) {
     media = grl_media_video_new ();
-  } else if (g_hash_table_lookup (ht, RDF_TYPE_IMAGE)) {
+  } else if ((type_filter & GRL_TYPE_FILTER_IMAGE) &&
+             g_hash_table_lookup (ht, RDF_TYPE_IMAGE)) {
     media = grl_media_image_new ();
-  } else if (g_hash_table_lookup (ht, RDF_TYPE_ARTIST)) {
-    media = grl_media_container_new ();
-  } else if (g_hash_table_lookup (ht, RDF_TYPE_ALBUM)) {
-    media = grl_media_container_new ();
-  } else if (g_hash_table_lookup (ht, RDF_TYPE_CONTAINER)) {
-    media = grl_media_container_new ();
-  } else if (g_hash_table_lookup (ht, RDF_TYPE_FOLDER)) {
-    media = grl_media_container_new ();
+  } else {
+    media = grl_tracker_build_grilo_media_default (ht);
   }
 
   g_hash_table_destroy (ht);
